## Custom Assets Library Plugin
The Custom Assets Library Plugin (CALP) is the successor to EAL and EAR which converts packs to use the TaleWeave Interface.
Supplementary functionality like Auras, Effects, Filters, Transformations, Animations and Sound has been moved to the
new Custom Assets Library Plugin Integrated Extension (CALPIE) plugin.

## Installing With R2ModMan
This package is designed specifically for R2ModMan and Talespire. 
You can install them via clicking on "Install with Mod Manager" or using the r2modman directly.

## Player Usage
This plugin used for players allows TaleSpire to search for TaleWeaver content in the R2Modman Directory.
This will allow direct download of TaleWeaver packs via R2Modman.

Use of variants required CALPIE.

### Default Keyboard Shortcuts

CALP has a bunch of keyboard shortcuts for triggering additional functionality. The default keys can be reconfigured to other
keys by editing the R2ModMan configuration for the CALP plugin. Keyboard shortcuts are divided into two sub-sections: functionality
triggers and asset spawn modifeirs. 

### Asset Types

```
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Kind            | Armature  | Shader            | Transparency       | Hide               | Comments                                           |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Any             |    Yes    | From Asset Bundle | Supported          | Not Supported      | Applies to the armature meshes only                |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Creature        |    No     | TS/Creature       | Not Supported      | Supported          | This is the default for minis                      |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Audio           |    No     | TS/Creature       | Not Supported      | Supported          | Mini starts playing audio when place on board*     |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Auras           |    No     | From Asset Bundle | Supported          | Supported*         | Not yet implemented in CAPL*                       |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Effect          |    No     | From Asset Bundle | Supported          | Supported*         | Used to create minis with non-TS shaders*          |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Transform       |    No     | TS/Creature       | Not Supported      | Supported          | Not yet implemented in CAPL*                       |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Prop            |    No     | TS/Placeable      | Not Supported      | Supported          | Not yet implemented in CAPL                        |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Tile            |    No     | TS/Placable       | Not Supported      | Not Supported      | Not yet implemented in CAPL                        |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| (Multi) Slab    |    N/A    | TS/Placable       | Not Supported      | Not Supported      | Partially supported. Height bar does not work*     |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
| Encounter       |    N/A    | Based On Asset    | Based On Asset     | Supported          | Not yet implemented in CAPL*                       |
+-----------------+-----------+-------------------+--------------------+--------------------+----------------------------------------------------+
```

Functionality marked by ```*``` required CALPIE to function.

## Developer Usage (Asset Creators)

Inbuilt into this package is a Binary Index writer which will generate a TaleWeaver compliant index file mimicing the interface of TaleWeaver. This is incredibly experimental and is being worked on. With CALP, EAR compliant files can be converted to TW compliance. This allows asset creators to keep their current workflow.

After CALP runs, an `index.json` and `index` files will be generated with some re-organizing. In the root of `index.json` file will contain an `assetPackId` variable which can be used to copy over into TaleWeaver's Steamapps folder.

### Unregister Batch File

If you update an asset (e.g. custom mini) you may or may not need to re-register it. If the asset's info.txt file and portrait.png file remained the same, you can just copy your new asset over the old one and the next time TS runs, it will use the updated version. If you did modify the info.txt or portrait.png file or added new assets to the pack then you will need to re-register the pack. To make this process easier, the Unregister batch file is provdied. Double click on this batch file to return the pack to its unregistered condition and it will get re-registered when TS is started again.

### Tweaking Packs

For testing purpose it is not possible to tweak the pack's index.json file (which is basically core TS version of most info.txt properties). To do this the pack needs to have been registered (or pre-registered). If the setting to delete the index.json is not turned on, the index.json file will remain after registration. To tweak it, follow these steps:

```
1. Ensure that in the CALP R2ModMan config you have Build Index Mode set to rebuildIndexIfMissing.
2. Rename the Assets folder back to CustomData
3. Delete the index (but not the index.json) file. Do not delete the Portraits folder.
4. Edit the index.json file
5. Restart TS to re-register the pack. This process will recognize the fact that you have a index.json file present and use it instead.
```

### Register For Vanilla Batch File

To help content developers use their custom assets even on Vanilla Talespire (non-modded Talespire), CALP generates a RegisterForVanilla.Bat file in the root of each pack. Using this batch file creates a directory link to be created in the Taleweaver directory that points to the pack location. Talespire will be convinced that the content is in the Taleweaver folder even though it is actually elsewhere. In order to use this batch file, you will need to do the following:

```
1. Open up a Command Prompt using the Run As Administrator. A normal Command Prompt does not have permissions to make links.
2. Execute the batch file with the location of your Taleweaver directory. For example: RegisterForVanilla E:\Steam\steamapps\common\TaleSpire\Taleweaver
```

To remove the link, just go to the Talwweaver folder and delete the corresponding sub-folder (not the d71427a1-5535-4fa7-82d7-4ca1e75edbfd folder since that is the core TS folder).

### Info.Txt Files

Below is a list of the currently supported proeprties in the info.txt file. Any of the old EAR properties (which are not yet listed here) are accepted but will be ignored.

```
{
  "name": "Assasin",
  "prefab": "",
  "kind": "Effect",
  "category": "Creature",
  "groupName": "Human",
  "description": "Assasin",
  "tags": "Human,Rogue,Assasin",
  "author": "Lord Ashes",
  "version": "1.0",
  "comment": "Maximo source",
  "size": 1.0,
  "prefab": "laassasin01"
  "variants": [ "laassasin01_crouch", "laassasin01_fire", "laassasin01_death" ]
}
```

All properties are optional and will take the default if not provided. The "prefab" property allow using a different prefab name than the asset bundle name. If empty the
prefab will default to the asset bundle name.

### Asset Bundle Contents

When making a single asset with no variants then the prefab should have the same name as the asset bundle name, which should have the same name as the asset bundle file name.
CALP uses the file name to determine the prefab name unless the prefab property is set in which case it uses that instead. If this rule is not followed a warning will appear
on startup indicating that the Prime Direction has been violated.

When making asset which have variants, the single asset bundle should have multiple prefabs with one matching the asset bundle file name or the prefab property. The rest of
the prefabs can be named as desired and this is the name that is used in the variants property (which is a list) to automatically create the variants and link them to the
morph list (which is done by CALPIE).


## Binary index writer usage (BepInEx Dev)
the index writer as a supplied interface and method to write the pack. PackContent data structure does not use blob references or ECS upon initial construction. 
```CSharp
CustomAssetsLibrary.CustomAssetLib.generate(folder);
```
After completion, the WritePack method will convert it into the ECS Blob structure and write the index file into the modpack directory.

## Changelog
```
3.0.0: Custom Asset Library refactored and decoupled segments for resilience
3.0.0: Custom Asset Loader split from CAL, handles loading TW packs in r2modman directory
3.0.0: Custom Asset DTO (CAD) split from CAL, handles shared data for sub-components
3.0.0: Custom Asset Compiler split from CAL, handles Compiling index binaries
3.0.0: Custom Asset Library wraps Custom Asset Compiler for CALP compatability
2.7.0: Re-implemented variants feature
2.6.0: Working region indepepndent version of CALP. 
2.5.6: Bug fix for default mini orientation
2.5.5: More region settings fixed
2.5.2: Corrected dependencies to use SmartConvert dependency plugin instead of local SmartConvert library
2.5.1: More localization of region setting fixes.
2.4.0: Localization of region setting fix.
2.3.1: Corrected asset pack hierarchy. No plugin or library change.
2.3.0: Bug fix for Cyberpunk update
2.2.0: Bug fix for assets without info.txt and/or portrait.png
2.2.0: Improved registration logging
2.1.0: Added SmartJsonConvert to allow plugin to work in different locale regions without needing to switch region settings before running Talespire.
2.0.0: Moved supplementary functionality to CALPIE.
```

Shoutout to Hollofoxes [Patreons](https://www.patreon.com/HolloFox) recognising their
mighty contribution to his caffeine addiction:
- John Fuller
- [Tales Tavern](https://talestavern.com/) - MadWizard

Also checkout LordAshes [donation page](https://lordashes.ca/TalespireDonate/Donate.php)
